/* $Id: funcs.c,v 1.8 2002/11/27 01:56:40 cegger Exp $
******************************************************************************

   Raw storage area access default (direct memory access) implementation.

   Copyright(C) 2001 Brian S. Julin [bri@tull.umassp.edu]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include "storage-stubs-lib.h"
#include <string.h>

int storage_stubs_storage2mem(ggi_ll_ststate_t src, size_t xoffset,
			      size_t yoffset, size_t stride,
			      void *membuf, size_t memstride,
			      size_t linewidth, size_t numlines)
{
	if (stride == 0) stride = src->stride;

	xoffset += yoffset * stride;
	if (src->bitdepth < 8) { 
#warning TODO
		return -1;
	}	/* if */

	if ((src->bitdepth % 8) || (src->bitdepth > 32)) return -1;
	xoffset *= src->bitdepth >> 3;
	stride  *= src->bitdepth >> 3;
	linewidth  *= src->bitdepth >> 3;

	while (numlines--) {
		memcpy(membuf, (char *)src->read.addr + xoffset, linewidth);
		xoffset += stride;
		membuf  += memstride;
	}	/* while */

	return GGI_ACCESSFUNC_UNNEEDED | GGI_ACCESSFUNC_NOFLUSH;
}	/* storage_stubs_storage2mem */


int storage_stubs_mem2storage(ggi_ll_ststate_t dest, size_t xoffset,
			      size_t yoffset, size_t stride,
			      void *membuf, size_t memstride,
			      size_t linewidth, size_t numlines)
{
	if (stride == 0) stride = dest->stride;
	xoffset += yoffset * stride;
	if (dest->bitdepth < 8) { 
#warning TODO
		return -1;
	}	/* if */

	if ((dest->bitdepth % 8) || (dest->bitdepth > 32)) return -1;
	xoffset *= dest->bitdepth >> 3;
	stride  *= dest->bitdepth >> 3;
	linewidth  *= dest->bitdepth >> 3;

	while (numlines--) {
		memcpy((char *)dest->write.addr + xoffset, membuf, linewidth);
		xoffset += stride;
		membuf  += memstride;
	}	/* while */

	return GGI_ACCESSFUNC_UNNEEDED | GGI_ACCESSFUNC_NOFLUSH;
}	/* storage_stubs_mem2storage */


int storage_stubs_storage2storage(ggi_ll_ststate_t dest, size_t xoffset,
				  size_t yoffset, size_t stride,
				  ggi_ll_ststate_t src, 
				  ggi_ll_stops_t srcops,
				  size_t srcxoffset,
				  size_t srcyoffset, size_t srcstride,
				  size_t linewidth, size_t numlines)
{
	if (stride == 0) stride = dest->stride;
	if (srcstride == 0) srcstride = src->stride;

	xoffset += yoffset * stride;

	if (dest->bitdepth != src->bitdepth) return -1;
	if (dest->bitdepth < 8) { 
#warning TODO 
		return -1;
	}	/* if */

	if ((dest->bitdepth % 8) || (dest->bitdepth > 32)) return -1;

	if (srcops->storage2mem == storage_stubs_storage2mem) {

		/* Insider trading :-) */

		srcxoffset += srcyoffset * srcstride;
		xoffset *= dest->bitdepth >> 3;
		stride  *= dest->bitdepth >> 3;
		linewidth *= dest->bitdepth >> 3;
		srcxoffset *= src->bitdepth >> 3;
		srcstride  *= src->bitdepth >> 3;

		while (numlines--) {
			memcpy((char *)dest->write.addr + xoffset,
			       (char *)src->read.addr + srcxoffset, linewidth);
			xoffset += stride;
			srcxoffset  += srcstride;
		}	/* while */
	} else {
		int rc;
		xoffset *= dest->bitdepth >> 3;
		stride  *= dest->bitdepth >> 3;

		rc = srcops->storage2mem(src, srcxoffset, srcyoffset, stride,
					 (char *)dest->write.addr + xoffset, 
					 stride, linewidth, numlines);
		if (rc <= 0) return rc;
	}	/* if */

	return GGI_ACCESSFUNC_UNNEEDED | GGI_ACCESSFUNC_NOFLUSH;
}	/* storage_stubs_storage2storage */


int storage_stubs_const2storage(ggi_ll_ststate_t dest, size_t xoffset,
				size_t yoffset, size_t stride,
				ggi_storageval val, size_t linewidth, 
				size_t numlines)
{
	if (stride == 0) stride = dest->stride;

	xoffset += yoffset * stride;

	switch (dest->bitdepth) {
	case 1:
#warning TODO
		break;

	case 2:
#warning TODO
		break;

	case 4:
#warning TODO
		break;

	case 8:
		while (numlines--) {
			memset(dest->write.addr + xoffset, val, linewidth);
			xoffset += stride;
		}	/* while */
		break;

	case 16:
		while (numlines--) {
			yoffset = linewidth;
			while (yoffset--) {
				*(((u_int16_t *)dest->write.addr + 
				   (yoffset * stride)) + xoffset) = val;
			}	/* while */
			xoffset += stride;
		}	/* while */
		break;

	case 32:
		while (numlines--) {
			yoffset = linewidth;
			while (yoffset--) {
				*(((u_int32_t *)dest->write.addr + 
				   (yoffset * stride)) + xoffset) = val;
			}	/* while */
			xoffset += stride;
		}	/* while */
		break;
	}	/* switch */

	return GGI_ACCESSFUNC_UNNEEDED | GGI_ACCESSFUNC_NOFLUSH;
}	/* Buf_storage_stubs_BufConst2Buf */







